(* ::Package:: *)

(* :Title: WriteRead *)
(* :Context: FileAccess`WriteRead` *)
(* :Author: Janko Slavic *)
(* :Summary: 
    This is the basic package form reading and writing to a file.
*)
(* :Package Version: 1.0 *)
(* :Mathematica Version: 4.2 *)
(* :History:
	1.0 by Janko Slavic, 2003.
*)
(* :Keywords: write, read, file.  *)

BeginPackage["FileAccess`WriteRead`"]


ReadConstant::usage="ReadConstant[stream, Name]
Reads the constant 'Name' from the fileStream 'stream'.";

WriteLine::usage="WriteLine[stream, data1D]
Writes the 'data1D' which is of the form {,,,} into a 
FileStream as single line.";

WriteColumn::usage="WriteColumn[stream, data1D]
Writes the 'data1D' which is of the form {,,,} into a 
FileStream as a column.";

AppendTableToFile::usage="AppendTableToFile[fileName_, data2D_]
Writes the 'data2D' which is of the form {{},{},{}} into a 
file, defined by the fileName.";

AppendTextToFile::usage="AppendTextToFile[fileName, text]
Writes the string 'text' to the file 'fileName'.";

AppendConstantToFile::usage="AppendConstantToFile[fileName, Name, Value] 
Writes the string 'Name' to the file 'fileName' and appends the 'Value' as a string.";

GetFileName::usage="GetFileName[fileName]
Extracts just the name of the file (it removes the path and extension).
";

Begin["`Private`"]

ReadConstant[stream_, Name_] := 
  Module[{tmp},
    tmp = Find[stream, Name];
    If[Head[tmp]==Symbol,
        0,0,
        Read[
          StringToStream[
            StringDrop[
              StringReplace[tmp, "," -> "."], StringLength[Name]
            ]
          ], Number
        ]
     ]
  ]

WriteLine[stream_, data1D_] :=
  Module[{i},
    For[i := 1, i <= Length[data1D], i++, 
      WriteString[stream, 
          ToString[FortranForm[N[data1D[[i]]]]] <> "\t"];];
    WriteString[stream, "\n"];
    ]
WriteColumn[stream_, data1D_] :=
  Module[{i},
    For[i := 1, i <= Length[data1D], i++, 
        WriteString[stream, 
            ToString[FortranForm[N[data1D[[i]]]]] <> "\n"];];
    ]
AppendTableToFile[fileName_, data2D_] := 
  Module[{stream},
    stream = OpenAppend[fileName];
    If[Length[data2D[[1]]] >= 2,
      For[j := 1, j <= Length[data2D], j++, 
        WriteLine[stream, data2D[[j]]]],
      WriteColumn[stream, data2D]];
    Close[stream];
    ]
AppendTextToFile[fileName_, text:_] :=
  Module[{stream,i},
    stream = OpenAppend[fileName];
    If[Length[text]==0,
      WriteString[stream, text<>"\n"];,
      For[i:=1,i<=Length[text],i++,
        WriteString[stream, text[[i]]<>"\n"];
      ];
    ];
    Close[stream];
  ]

AppendConstantToFile[fileName_, Name_, Value_] :=
  Module[{stream,i},
    stream = OpenAppend[fileName];
    WriteString[stream, Name<>ToString[FortranForm[N[Value]]] <> "\n"];
    Close[stream];
  ]

GetFileName[fileName_]:=
  Module[{tmp,pos},
    pos=Flatten[StringPosition[fileName,"/"]];
    StringTake[fileName,{pos[[Length[pos]]]+1,StringLength[fileName]-4}]
    ]

End[]

Protect[ReadConstant,WriteLine,WriteColumn,AppendTableToFile,AppendTextToFile,AppendConstantToFile,GetFileName]

EndPackage[]
