(* ::Package:: *)

(* :Title: Common *)
(* :Context: MultiBodyDynamics`Common` *)
(* :Author: Janko Slavic *)
(* :Summary: 
    Miscellaneous tools for MultiBodyDynamics.
*)
(* :Package Version: 1.01 *)
(* :Mathematica Version: 4.2, 7.0 *)
(* :History:
	1.01 by Janko Slavic, Aug 2009.
	1.0 by Janko Slavic, 2003.
*)
(* :Keywords: GetKinematics.  
*)


BeginPackage["MultiBodyDynamics`Common`"]

GetKinematics::usage = "GetKinematics[a0,v0,x0,a1,dt]:=
If the initial acceleration, velocity and displacement at time 0:
a0, v0 and x0 are given, and furthermore are give the
acceleration at time 1: a1 and the time difference dt,
then the velocity and displacement can be calculater.\n
The output is of the form:\n
{a1, v1, x1}
";

GetKinematicsDisContinuous::usage = "GetKinematics[a0,v0,x0,v1,a1,dt]:=
Similar to the function GetKinematics, but velocity v1 is given.\n
The output is of the form:\n
{a1, v1, x1}
";


Begin["`Private`"]

GetKinematics[a0_,v0_,x0_,a1_,dt_]:=
  Module[{v1,i},
    v1=v0+(a0+a1)dt/2;
    {N[a1],N[v1],N[x0+(v0+v1)dt/2]}
    ]

GetKinematicsDisContinuous[a0_,v0_,x0_,v1_,a1_,dt_]:=
  Module[{i},
    {N[a1],N[v1],N[x0+(v0+v1)dt/2]}
    ]

          
End[]

Protect[GetKinematics,GetKinematicsDisContinuous]

EndPackage[]
