(* ::Package:: *)

(* :Title: MultiBodyDynamics *)
(* :Context: MultiBodyDynamics`MultiBodyDynamics` *)
(* :Author: Janko Slavic *)
(* :Summary: 
    Packages for working with MultiBodyDynamics simulator.
*)
(* :Package Version: 1.02 *)
(* :Mathematica Version: 5.1, 7.0 *)
(* :History:
    1.02 by Janko Slavic, Aug 2009.
        updated for Mathematica 7.0. 
	1.01 by Janko Slavic, feb 2005.
		moved from Misc`MultiBodyDynamics to MultiBodyDynamics`MultiBodyDynamics
	1.0 by Janko Slavic, 2003.
*)
(* :Keywords: mbd.  
*)


BeginPackage["MultiBodyDynamics`MultiBodyDynamics`",{"MultiBodyDynamics`FileSystem`","Mechanics`AnalyticalMechanics`"}]

GetNames::usage="GetNames[definition]
Gets the names out of a definition, example Definition[DC].";

JacobianMatrix::usage="JacobianMatrix[vector, base]
Returns the Jacobian matrix of the vector according to the base.";

GetJandj::usage="GetJandj[dependentCoordinates,generalizedCoordinates]
Returns the vector J and the scalar j for each dependent cooridnate.\n
Output form: {{J1,j1},{J2,j2},...}";

PrepareDependentCoordinates::usage="
PrepareDependentCoordinates[definitionOfDependentCoord,
dependentCoordinates,generalizedCoordinates, time]
Prepares the the dependent coordinates file.\n
Input: defOfDepCoord - Definition[dependentCoordinates],
dependentCoordinates, generalizedCoordinates.\n
Output: List
";


Begin["`Private`"]

  
GetNames[definition_]:=Module[{a,start,end,strPos,out,i,tmp},
    start=StringPosition[ToString[definition],"{"][[1,1]];
    end=StringPosition[ToString[definition],"}"][[1,1]];    
    a=StringTake[ToString[definition],{start+1,end-1}];
    strPos=StringPosition[a,", "];
    If[Length[strPos]==0,out={a},
    (*begin get names*)
      out={StringTake[a,strPos[[1,1]]-1]};
      For[i:=1,i<Length[strPos],i++,
        tmp=StringTake[
            a,{strPos[[i,2]]+1,strPos[[i+1,1]]-1}];
        AppendTo[out,tmp];
        ];
      tmp=StringTake[
          a,{strPos[[Length[strPos],2]]+1,StringLength[a]}];
      AppendTo[out,tmp];
    ];(*end get names*)
    out
    ]
    
JacobianMatrix[vector_,base_]:=Module[{i,j},
    Table[
      D[vector[[j]],base[[i]]],
      {j,1,Length[vector]},
      {i,1,Length[base]}
      ]
    ]
    
GetJandj[dependentCoordinates_,generalizedCoordinates_,time_]:=Module[{J,j,i,out},
    J[i_]:=
      JacobianMatrix[{dependentCoordinates[[i]]},generalizedCoordinates];
    j[i_]:=Simplify[D[dependentCoordinates[[i]],{time,2}]-Dot[J[i],D[generalizedCoordinates,{time,2}]]];
    out={{J[1],j[1]}};
    For[i:=2,i<=Length[dependentCoordinates],i++,
      AppendTo[out,{J[i],j[i]}]
      ];
    out
    ]    
    
    
PrepareDependentCoordinates[defOfDepCoord_,depCoord_,genCoord_,time_]:=
    Module[{names,Jandj,out,i},
      If[StringLength[ToString[defOfDepCoord]]==8,
        (*no data*)
        out={{""}},
        (*is data*)
	      names:=GetNames[defOfDepCoord];
 	      Jandj:=GetJandj[depCoord,genCoord,time];
  	      out={{"%%%new%%%",
            names[[1]],
            "%%definition",
            depCoord[[1]],
            D[depCoord[[1]],{time,1}],
            D[depCoord[[1]],{time,2}],            
            "%%J",
            Flatten[Jandj[[1,1,1]]],
            "%%j",
            Flatten[Jandj[[1,2]]]}," "};
          For[i:=2,i<=Length[depCoord],i++,
           AppendTo[out,
              {"%%%new%%%",
                names[[i]],
                "%%definition",
                depCoord[[i]],
                D[depCoord[[i]],{time,1}],
                D[depCoord[[i]],{time,2}],            
                "%%J",
                Flatten[Jandj[[i,1,1]]],
                "%%j",
                Flatten[Jandj[[i,2]]]," "}
           ];
          ];
        ];(*end if*)
         Flatten[out]
      ];    
      
End[]

Protect[GetNames,JacobianMatrix,GetJandj,PrepareDependentCoordinates]

EndPackage[]
