(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 7.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[     11014,        363]
NotebookOptionsPosition[      9064,        298]
NotebookOutlinePosition[      9590,        317]
CellTagsIndexPosition[      9547,        314]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Libraries", "Subtitle",
 CellChangeTimes->{{3.458361508258*^9, 3.458361510667*^9}}],

Cell[" Path to custom libraries", "Text",
 CellChangeTimes->{{3.4583615146099997`*^9, 3.458361532458*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"AppendTo", "[", 
   RowBox[{"$Path", ",", "\"\<j:/Work Janko/CommonFiles/Math\>\""}], "]"}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Needs", "[", "\"\<MultiBodyDynamics`MultiBodyDynamics`\>\"", "]"}],
   ";"}]], "Input",
 CellChangeTimes->{{3.4583609059560003`*^9, 3.458360915384*^9}, {
  3.458361199059*^9, 3.458361200579*^9}, {3.458362281135*^9, 
  3.458362290453*^9}, {3.4583628489040003`*^9, 3.45836284937*^9}, {
  3.45836344249*^9, 3.458363445101*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["2 Mass System", "Title",
 CellChangeTimes->{{3.458363490692*^9, 3.458363496554*^9}, {3.458369733508*^9,
   3.4583697810620003`*^9}, {3.4583703082530003`*^9, 3.458370310291*^9}, {
  3.4583710860030003`*^9, 3.458371089193*^9}, {3.458460533269*^9, 
  3.458460540178*^9}}],

Cell["\<\
dr. Janko Slavic, 2009
www.fs.uni-lj.si/ladisk/~slavic/\
\>", "Subsubtitle",
 CellChangeTimes->{{3.458361595974*^9, 3.458361638626*^9}}],

Cell[CellGroupData[{

Cell["Coordinates", "Subtitle",
 CellChangeTimes->{{3.458361389211*^9, 3.458361391613*^9}}],

Cell["Generalized coordinates", "Text",
 CellChangeTimes->{{3.458369878052*^9, 3.4583698794849997`*^9}, {
  3.4583711271429996`*^9, 3.458371129126*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"GC", ":=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"b`mass1`x", "[", "t", "]"}], ",", 
     RowBox[{"b`mass1`y", "[", "t", "]"}], ",", 
     RowBox[{"b`mass1`Rz", "[", "t", "]"}]}], "}"}]}], ";"}]], "Input"],

Cell["Dependent coordinates", "Text",
 CellChangeTimes->{3.458460579301*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"DC", ":=", 
   RowBox[{"{", "}"}]}], ";"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Kinetic, potential energy and virtual work", "Subtitle",
 CellChangeTimes->{{3.4583614035880003`*^9, 3.4583614584189997`*^9}}],

Cell["Kinetic energy", "Text",
 CellChangeTimes->{{3.458361363825*^9, 3.458361368514*^9}, {3.458361414918*^9,
   3.458361415526*^9}, {3.4583615517679996`*^9, 3.458361553936*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Ek", "[", "t_", "]"}], ":=", "\[IndentingNewLine]", 
  RowBox[{"(*", "r1", "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], "b`mass1`m", "*", 
    SuperscriptBox[
     RowBox[{"vr1", "[", "t", "]"}], "2"]}], "+", 
   RowBox[{
    FractionBox["1", "2"], "b`mass1`Jzz", "*", 
    SuperscriptBox[
     RowBox[{
      RowBox[{"b`mass1`Rz", "'"}], "[", "t", "]"}], "2"]}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"vr1", "[", "t_", "]"}], ":=", 
  SqrtBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{
      RowBox[{"b`mass1`x", "'"}], "[", "t", "]"}], "2"], "+", 
    SuperscriptBox[
     RowBox[{
      RowBox[{"b`mass1`y", "'"}], "[", "t", "]"}], "2"]}]]}]], "Input"],

Cell["Potential energy", "Text",
 CellChangeTimes->{{3.4583703530369997`*^9, 3.4583703591619997`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Ep", "[", "t_", "]"}], ":=", 
  RowBox[{"b`mass1`m", " ", "c`gravity", " ", 
   RowBox[{"b`mass1`y", "[", "t", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.4584607083*^9, 3.4584607090299997`*^9}}],

Cell["Virtual work", "Text",
 CellChangeTimes->{{3.458460609598*^9, 3.458460611559*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Delta]W", "[", "t_", "]"}], ":=", "0"}]], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Equations of motion", "Subtitle",
 CellChangeTimes->{{3.458361475104*^9, 3.458361478633*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"EM", "=", 
   RowBox[{"EquationsOfMotion", "[", 
    RowBox[{
     RowBox[{"Ek", "[", "t", "]"}], ",", 
     RowBox[{"Ep", "[", "t", "]"}], ",", 
     RowBox[{"GeneralizedForces", "[", 
      RowBox[{
       RowBox[{"\[Delta]W", "[", "t", "]"}], ",", "GC"}], "]"}], ",", "GC", 
     ",", "t"}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"m", ",", "h"}], "}"}], "=", 
   RowBox[{"MassMatrixAndHVector", "[", 
    RowBox[{
     RowBox[{"Ek", "[", "t", "]"}], ",", 
     RowBox[{"Ep", "[", "t", "]"}], ",", 
     RowBox[{"GeneralizedForces", "[", 
      RowBox[{
       RowBox[{"\[Delta]W", "[", "t", "]"}], ",", "GC"}], "]"}], ",", "GC", 
     ",", "t"}], "]"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MatrixForm", "[", "m", "]"}]], "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"b`mass1`m", "0", "0"},
     {"0", "b`mass1`m", "0"},
     {"0", "0", "b`mass1`Jzz"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.458460714659*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MatrixForm", "[", "h", "]"}]], "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", 
   TagBox[GridBox[{
      {"0"},
      {
       RowBox[{
        RowBox[{"-", "c`gravity"}], " ", "b`mass1`m"}]},
      {"0"}
     },
     GridBoxAlignment->{
      "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
        "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}, "RowsIndexed" -> {}}],
    Column], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.458460714686*^9}]
}, Open  ]],

Cell["Name of model, folder name", "Text",
 CellChangeTimes->{{3.458361771413*^9, 3.458361774559*^9}, {3.458370441619*^9,
   3.458370446134*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"filename", "=", "\"\<2Mass\>\""}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"folder", "=", "\"\<j:/\>\""}], ";"}]}], "Input",
 CellChangeTimes->{{3.458370419329*^9, 3.458370440045*^9}, {3.458371188915*^9,
   3.458371193196*^9}, {3.458460641576*^9, 3.4584606425889997`*^9}}],

Cell["Write mass matrix, h vector and mechanical energy", "Text",
 CellChangeTimes->{{3.458361787881*^9, 3.4583617967*^9}, {3.458361875633*^9, 
  3.458361880991*^9}}],

Cell[BoxData[{
 RowBox[{"MMXWrite", "[", 
  RowBox[{
   RowBox[{"folder", "<>", "filename", "<>", "\"\<.mmx\>\""}], ",", "m"}], 
  "]"}], "\n", 
 RowBox[{"HVWrite", "[", 
  RowBox[{
   RowBox[{"folder", "<>", "filename", "<>", "\"\<.hv\>\""}], ",", "h"}], 
  "]"}], "\n", 
 RowBox[{"HVWrite", "[", 
  RowBox[{
   RowBox[{"folder", "<>", "filename", "<>", "\"\<.me\>\""}], ",", 
   RowBox[{
    RowBox[{"Ek", "[", "t", "]"}], "+", 
    RowBox[{"Ep", "[", "t", "]"}]}]}], "]"}]}], "Input",
 CellChangeTimes->{
  3.458370453586*^9, {3.458370494225*^9, 3.4583704999519997`*^9}}],

Cell["Write generalized and dependent coordinates", "Text",
 CellChangeTimes->{{3.4583619063640003`*^9, 3.4583619155769997`*^9}}],

Cell[BoxData[{
 RowBox[{"GCWrite", "[", 
  RowBox[{
   RowBox[{"folder", "<>", "filename", "<>", "\"\<.gc\>\""}], ",", "GC"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"GCWrite", "[", 
  RowBox[{
   RowBox[{"folder", "<>", "filename", "<>", "\"\<.dc\>\""}], ",", 
   RowBox[{"PrepareDependentCoordinates", "[", 
    RowBox[{
     RowBox[{"Definition", "[", "DC", "]"}], ",", "DC", ",", "GC", ",", "t"}],
     "]"}]}], "]"}]}], "Input",
 CellChangeTimes->{3.4583705016400003`*^9, 3.458370938348*^9}],

Cell["Define and write constants", "Text",
 CellChangeTimes->{{3.4583619210039997`*^9, 3.458361930696*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"c`gravity", "=", "9.81"}], ";"}]], "Input",
 CellChangeTimes->{{3.4584606537720003`*^9, 3.458460654487*^9}}],

Cell[BoxData[
 RowBox[{"CNSWrite", "[", 
  RowBox[{
   RowBox[{"folder", "<>", "filename", "<>", "\"\<.cns\>\""}], ",", 
   RowBox[{"Names", "[", "\"\<c`*\>\"", "]"}], ",", 
   RowBox[{"ToExpression", "[", 
    RowBox[{"Names", "[", "\"\<c`*\>\"", "]"}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{3.4585332518450003`*^9}]
}, Open  ]]
}, Open  ]]
},
PrintingStyleEnvironment->"Condensed",
WindowSize->{971, 681},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
PrintingCopies->1,
PrintingPageRange->{Automatic, Automatic},
FrontEndVersion->"7.0 for Microsoft Windows (32-bit) (November 10, 2008)",
StyleDefinitions->FrontEnd`FileName[{"Creative"}, "PastelColor.nb", 
  CharacterEncoding -> "WindowsEastEurope"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[567, 22, 89, 1, 24, "Subtitle"],
Cell[659, 25, 106, 1, 26, "Text"],
Cell[768, 28, 147, 4, 37, "Input"],
Cell[918, 34, 359, 7, 37, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1314, 46, 274, 4, 36, "Title"],
Cell[1591, 52, 146, 4, 47, "Subsubtitle"],
Cell[CellGroupData[{
Cell[1762, 60, 91, 1, 24, "Subtitle"],
Cell[1856, 63, 152, 2, 26, "Text"],
Cell[2011, 67, 269, 7, 56, "Input"],
Cell[2283, 76, 76, 1, 26, "Text"],
Cell[2362, 79, 88, 3, 37, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2487, 87, 132, 1, 24, "Subtitle"],
Cell[2622, 90, 178, 2, 26, "Text"],
Cell[2803, 94, 449, 13, 87, "Input"],
Cell[3255, 109, 291, 10, 49, "Input"],
Cell[3549, 121, 102, 1, 29, "Text"],
Cell[3654, 124, 229, 5, 41, "Input"],
Cell[3886, 131, 88, 1, 29, "Text"],
Cell[3977, 134, 87, 2, 41, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[4101, 141, 99, 1, 36, "Subtitle"],
Cell[4203, 144, 355, 10, 41, "Input"],
Cell[4561, 156, 406, 12, 41, "Input"],
Cell[CellGroupData[{
Cell[4992, 172, 63, 1, 41, "Input"],
Cell[5058, 175, 682, 19, 67, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[5777, 199, 63, 1, 41, "Input"],
Cell[5843, 202, 747, 23, 67, "Output"]
}, Open  ]],
Cell[6605, 228, 145, 2, 29, "Text"],
Cell[6753, 232, 319, 6, 62, "Input"],
Cell[7075, 240, 166, 2, 29, "Text"],
Cell[7244, 244, 574, 16, 82, "Input"],
Cell[7821, 262, 129, 1, 29, "Text"],
Cell[7953, 265, 500, 12, 56, "Input"],
Cell[8456, 279, 107, 1, 26, "Text"],
Cell[8566, 282, 143, 3, 37, "Input"],
Cell[8712, 287, 324, 7, 37, "Input"]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
