(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 7.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[     13181,        405]
NotebookOptionsPosition[     11554,        348]
NotebookOutlinePosition[     12080,        367]
CellTagsIndexPosition[     12037,        364]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Libraries", "Subtitle",
 CellChangeTimes->{{3.458361508258*^9, 3.458361510667*^9}}],

Cell[" Path to custom libraries", "Text",
 CellChangeTimes->{{3.4583615146099997`*^9, 3.458361532458*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"AppendTo", "[", 
   RowBox[{"$Path", ",", "\"\<j:/Work Janko/CommonFiles/Math\>\""}], "]"}], 
  ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Needs", "[", "\"\<MultiBodyDynamics`MultiBodyDynamics`\>\"", "]"}],
   ";"}]], "Input",
 CellChangeTimes->{{3.4583609059560003`*^9, 3.458360915384*^9}, {
  3.458361199059*^9, 3.458361200579*^9}, {3.458362281135*^9, 
  3.458362290453*^9}, {3.4583628489040003`*^9, 3.45836284937*^9}, {
  3.45836344249*^9, 3.458363445101*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["WoodpeckerToy - Bird", "Title",
 CellChangeTimes->{{3.458363490692*^9, 3.458363515158*^9}, {3.458365276576*^9,
   3.458365278711*^9}}],

Cell["\<\
dr. Janko Slavic, 2009
www.fs.uni-lj.si/ladisk/~slavic/\
\>", "Subsubtitle",
 CellChangeTimes->{{3.458361595974*^9, 3.458361638626*^9}}],

Cell["This file is used to create the exact shape of the bird", "Text",
 CellChangeTimes->{{3.45836528169*^9, 3.4583652962*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Circl", "[", 
   RowBox[{"r_", ",", "n_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"a", ",", "i"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"r", " ", 
         RowBox[{"Cos", "[", 
          RowBox[{"2", " ", "\[Pi]", " ", 
           FractionBox["i", "n"]}], "]"}]}], ",", 
        RowBox[{"r", " ", 
         RowBox[{"Sin", "[", 
          RowBox[{"2", " ", "\[Pi]", " ", 
           FractionBox["i", "n"]}], "]"}]}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "0", ",", "n"}], "}"}]}], "]"}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Elipse", "[", 
   RowBox[{"r1_", ",", "r2_", ",", "n_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"a", ",", "i"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"r1", " ", 
         RowBox[{"Cos", "[", 
          RowBox[{"2", " ", "\[Pi]", " ", 
           FractionBox["i", "n"]}], "]"}]}], ",", 
        RowBox[{"r2", " ", 
         RowBox[{"Sin", "[", 
          RowBox[{"2", " ", "\[Pi]", " ", 
           FractionBox["i", "n"]}], "]"}]}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "0", ",", "n"}], "}"}]}], "]"}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"body", "=", 
   RowBox[{"Elipse", "[", 
    RowBox[{"0.01", ",", "0.02", ",", "50"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"beak", "=", 
   RowBox[{"Elipse", "[", 
    RowBox[{"0.0201", ",", "0.002", ",", "50"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"head", "=", 
   RowBox[{"Elipse", "[", 
    RowBox[{"0.01", ",", "0.01", ",", "50"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"feet", "=", 
   RowBox[{"Elipse", "[", 
    RowBox[{"0.015", ",", "0.002", ",", "50"}], "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.458369203755*^9, 3.458369211244*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"TranslateData", "[", 
   RowBox[{"data_", ",", "x_", ",", "y_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"Transpose", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"x", ",", "y"}], "}"}], "+", 
      RowBox[{"Transpose", "[", "data", "]"}]}], "]"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.458369215793*^9, 3.4583692162799997`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"data", "=", 
   RowBox[{"Join", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Take", "[", 
      RowBox[{"body", ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "6"}], "}"}]}], "]"}], ",", "\[IndentingNewLine]", 
     RowBox[{"TranslateData", "[", 
      RowBox[{
       RowBox[{"Take", "[", 
        RowBox[{"head", ",", 
         RowBox[{"{", 
          RowBox[{"45", ",", "50"}], "}"}]}], "]"}], ",", "0", ",", "0.02"}], 
      "]"}], ",", "\[IndentingNewLine]", 
     RowBox[{"TranslateData", "[", 
      RowBox[{
       RowBox[{"Take", "[", 
        RowBox[{"head", ",", 
         RowBox[{"{", 
          RowBox[{"1", ",", "20"}], "}"}]}], "]"}], ",", "0", ",", "0.02"}], 
      "]"}], ",", "\[IndentingNewLine]", 
     RowBox[{"TranslateData", "[", 
      RowBox[{
       RowBox[{"Take", "[", 
        RowBox[{"beak", ",", 
         RowBox[{"{", 
          RowBox[{"17", ",", "35"}], "}"}]}], "]"}], ",", "0", ",", "0.02"}], 
      "]"}], ",", "\[IndentingNewLine]", 
     RowBox[{"TranslateData", "[", 
      RowBox[{
       RowBox[{"Take", "[", 
        RowBox[{"head", ",", 
         RowBox[{"{", 
          RowBox[{"31", ",", "32"}], "}"}]}], "]"}], ",", "0", ",", "0.02"}], 
      "]"}], ",", "\[IndentingNewLine]", 
     RowBox[{"TranslateData", "[", 
      RowBox[{
       RowBox[{"Take", "[", 
        RowBox[{"body", ",", 
         RowBox[{"{", 
          RowBox[{"21", ",", "25"}], "}"}]}], "]"}], ",", "0", ",", "0"}], 
      "]"}], ",", "\[IndentingNewLine]", 
     RowBox[{"TranslateData", "[", 
      RowBox[{
       RowBox[{"Take", "[", 
        RowBox[{"feet", ",", 
         RowBox[{"{", 
          RowBox[{"20", ",", "32"}], "}"}]}], "]"}], ",", "0", ",", "0"}], 
      "]"}], ",", "\[IndentingNewLine]", 
     RowBox[{"TranslateData", "[", 
      RowBox[{
       RowBox[{"Take", "[", 
        RowBox[{"body", ",", 
         RowBox[{"{", 
          RowBox[{"27", ",", "51"}], "}"}]}], "]"}], ",", "0", ",", "0"}], 
      "]"}]}], "\[IndentingNewLine]", "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.4583692200810003`*^9, 3.458369264115*^9}, {
  3.458369305016*^9, 3.458369305566*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ListPlot", "[", 
  RowBox[{"data", ",", 
   RowBox[{"Frame", "\[Rule]", "True"}], ",", 
   RowBox[{"Axes", "\[Rule]", "False"}], ",", 
   RowBox[{"PlotJoined", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellChangeTimes->{3.458369270847*^9}],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {Hue[0.67, 0.6, 0.6], LineBox[CompressedData["
1:eJxdVHtMU2cUb1BMeBUzyGSCDaQhQ/APn4xNyUFB8F0eAxShiAEXt6jMSdDy
WsO7tqXtvQXl5Qtn4pIqIFqC4SjMoK7RmAymM8qqq6KSgmwag1M24z3fH99N
bm5Ozved8zvn/n6/sJ37Ugs8ZDLZjv/fD9+SQFuSU1MFMulRH7Pdrk+pguai
0G336/JhWYp3cuRIJczOU539JvcAuLa+Gr3gXQnj47aOsIhymF9ieq6e0kLt
9ZmfDX2V4N/Z/pVo/hEM7gueOwLqoHJX+pqb98vg/cjVfwcmdSx/vlOr/2Rc
z+53JWT3vp3XwOo/jA4KeNBjYv2zIvJT/3GbgfCF+YVY0pYKQPg/fkWWHw6O
e7d30Mrut8rerLo1p4nVD5qKL3XBEda/bY6t65fdRxk+RXLw+t6KZoa/OPbt
3LyyFlB+vfX9aLAGFgnDr/vUraCut39mmCoCt2swYlVoG2T1RlvbC7+H1Y0n
9/3W3wbrWhTbjcm7IfW4j3Z2TDtUW+oPZ3arYeBaTtDj+naAOzZlnJAAzmYf
W3g3i5Fi6TzSeakeUj2pH1I/CQ8SHgkvEl5pHqR5ApPsG1/WaHFEPBiyNr0R
yofMdzvkNbjIZ5PcsKIRjkRVZG2M1aFnXside/JG0AYXqzw8jKh5pjw15LRC
+OmVP9z4yYStaZElvues4Fg43ZPpZcGxBOfComIrVKQ7KgKVAt6Km1E8WmmF
FnNsjnFCwJ545Wj8OxE+7RoYOvydiJ+v/zWm6bII/YuPRgZZRaT/Sfnbo6aM
qC9Fdn+1K+Z8+RuB1Y8Yyy78s1dg/eV/aQo2lwsMX9bwksCLiQLDXwox49MB
Apvv5RD0jz2xsPlvLpirXXHFwvYTKj7eX3bcQvxA4i/tk+M3Ev8lfiHpQ+If
kn4kfiLpS+Ivkv4e+V7p/D2/Bv9W5Vwq2JsBTV7uTPUfdeg9NfhF95o0OGO/
m2t8ocMOv12JpRYVZHhOqvxNeszQhT+fH7oOHIf8D8JVA0Y/Pe1O3RQHfhPV
l7ZojOiM3JxUumcx2BXVL1LsRiT9c3nk7iNXH7n+yOFDDj9y8yE3P3L7QW5/
yO0Xaf+Tp5aHz3jpkfh+bbrt8rcNDUh6cJ2MGujZYEbSi37WM9WTWguSnmzz
+vYnagQkvTXszD0xSyEi6dHh2+eoK2QxUEx6pvOkd6pHfkD9yC8ID/kJ4SW/
oXk4P0XOb5HzY+T8Gjk/R97v/wNAq+6R
     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  AxesOrigin->{0, 0},
  Frame->True,
  PlotRange->{{-0.0201, 0.01}, {-0.019960534568565433`, 
   0.029980267284282717`}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{{3.458369274026*^9, 3.458369306957*^9}, 
   3.4583693946140003`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a", "=", 
  RowBox[{"Min", "[", 
   RowBox[{
    RowBox[{"Transpose", "[", "data", "]"}], "[", 
    RowBox[{"[", "1", "]"}], "]"}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"-", "0.0201`"}]], "Output",
 CellChangeTimes->{{3.458369274058*^9, 3.458369308552*^9}, 3.45836939469*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Position", "[", 
  RowBox[{"data", ",", "a"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{"42", ",", "1"}], "}"}], "}"}]], "Output",
 CellChangeTimes->{{3.458369274079*^9, 3.4583693095959997`*^9}, 
   3.458369394743*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"data", "[", 
  RowBox[{"[", 
   RowBox[{"42", ",", "2"}], "]"}], "]"}]], "Input"],

Cell[BoxData["0.02`"], "Output",
 CellChangeTimes->{{3.4583692741879997`*^9, 3.458369310517*^9}, 
   3.458369394802*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Write/ReadFile", "Subtitle"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"filename", "=", "\"\<j:/bird.exs\>\""}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"EXSWrite", "[", 
  RowBox[{"filename", ",", "data"}], "]"}]}], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dataread", "=", 
   RowBox[{"EXSRead", "[", "filename", "]"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ListPlot", "[", "dataread", "]"}]], "Input",
 CellChangeTimes->{3.458369281583*^9}],

Cell[BoxData[
 GraphicsBox[
  {Hue[0.67, 0.6, 0.6], PointBox[CompressedData["
1:eJxdVHtMU2cUb1BMeBUzyGSCDaQhQ/APn4xNyUFB8F0eAxShiAEXt6jMSdDy
WsO7tqXtvQXl5Qtn4pIqIFqC4SjMoK7RmAymM8qqq6KSgmwag1M24z3fH99N
bm5Ozved8zvn/n6/sJ37Ugs8ZDLZjv/fD9+SQFuSU1MFMulRH7Pdrk+pguai
0G336/JhWYp3cuRIJczOU539JvcAuLa+Gr3gXQnj47aOsIhymF9ieq6e0kLt
9ZmfDX2V4N/Z/pVo/hEM7gueOwLqoHJX+pqb98vg/cjVfwcmdSx/vlOr/2Rc
z+53JWT3vp3XwOo/jA4KeNBjYv2zIvJT/3GbgfCF+YVY0pYKQPg/fkWWHw6O
e7d30Mrut8rerLo1p4nVD5qKL3XBEda/bY6t65fdRxk+RXLw+t6KZoa/OPbt
3LyyFlB+vfX9aLAGFgnDr/vUraCut39mmCoCt2swYlVoG2T1RlvbC7+H1Y0n
9/3W3wbrWhTbjcm7IfW4j3Z2TDtUW+oPZ3arYeBaTtDj+naAOzZlnJAAzmYf
W3g3i5Fi6TzSeakeUj2pH1I/CQ8SHgkvEl5pHqR5ApPsG1/WaHFEPBiyNr0R
yofMdzvkNbjIZ5PcsKIRjkRVZG2M1aFnXside/JG0AYXqzw8jKh5pjw15LRC
+OmVP9z4yYStaZElvues4Fg43ZPpZcGxBOfComIrVKQ7KgKVAt6Km1E8WmmF
FnNsjnFCwJ545Wj8OxE+7RoYOvydiJ+v/zWm6bII/YuPRgZZRaT/Sfnbo6aM
qC9Fdn+1K+Z8+RuB1Y8Yyy78s1dg/eV/aQo2lwsMX9bwksCLiQLDXwox49MB
Apvv5RD0jz2xsPlvLpirXXHFwvYTKj7eX3bcQvxA4i/tk+M3Ev8lfiHpQ+If
kn4kfiLpS+Ivkv4e+V7p/D2/Bv9W5Vwq2JsBTV7uTPUfdeg9NfhF95o0OGO/
m2t8ocMOv12JpRYVZHhOqvxNeszQhT+fH7oOHIf8D8JVA0Y/Pe1O3RQHfhPV
l7ZojOiM3JxUumcx2BXVL1LsRiT9c3nk7iNXH7n+yOFDDj9y8yE3P3L7QW5/
yO0Xaf+Tp5aHz3jpkfh+bbrt8rcNDUh6cJ2MGujZYEbSi37WM9WTWguSnmzz
+vYnagQkvTXszD0xSyEi6dHh2+eoK2QxUEx6pvOkd6pHfkD9yC8ID/kJ4SW/
oXk4P0XOb5HzY+T8Gjk/R97v/wNAq+6R
    "]]},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->True,
  PlotRange->Automatic,
  PlotRangeClipping->True]], "Output",
 CellChangeTimes->{{3.458369290439*^9, 3.458369314425*^9}, 3.458369394907*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
PrintingStyleEnvironment->"Condensed",
WindowSize->{971, 709},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
PrintingCopies->1,
PrintingPageRange->{Automatic, Automatic},
FrontEndVersion->"7.0 for Microsoft Windows (32-bit) (November 10, 2008)",
StyleDefinitions->FrontEnd`FileName[{"Creative"}, "PastelColor.nb", 
  CharacterEncoding -> "WindowsEastEurope"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[567, 22, 89, 1, 36, "Subtitle"],
Cell[659, 25, 106, 1, 29, "Text"],
Cell[768, 28, 147, 4, 41, "Input"],
Cell[918, 34, 359, 7, 41, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1314, 46, 140, 2, 73, "Title"],
Cell[1457, 50, 146, 4, 49, "Subsubtitle"],
Cell[1606, 56, 128, 1, 29, "Text"],
Cell[1737, 59, 728, 22, 111, "Input"],
Cell[2468, 83, 744, 22, 111, "Input"],
Cell[3215, 107, 663, 20, 102, "Input"],
Cell[3881, 129, 457, 12, 62, "Input"],
Cell[4341, 143, 2160, 59, 222, "Input"],
Cell[CellGroupData[{
Cell[6526, 206, 262, 6, 41, "Input"],
Cell[6791, 214, 1723, 35, 245, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[8551, 254, 174, 5, 41, "Input"],
Cell[8728, 261, 129, 2, 40, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[8894, 268, 87, 2, 41, "Input"],
Cell[8984, 272, 186, 5, 40, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[9207, 282, 105, 3, 41, "Input"],
Cell[9315, 287, 120, 2, 40, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[9472, 294, 34, 0, 36, "Subtitle"],
Cell[9509, 296, 192, 5, 62, "Input"],
Cell[9704, 303, 116, 3, 41, "Input"],
Cell[CellGroupData[{
Cell[9845, 310, 107, 2, 41, "Input"],
Cell[9955, 314, 1559, 29, 241, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
